% Valuing the economic benefits of species recovery programmes

clear all
clc

global B_backup;


%% ****************************  loading data  ****************************


EstimOpt.DataFile = ('defra_data_02.mat');
DATA_prepare = load(EstimOpt.DataFile);

NCT = 12;
NAlt = 3;


%% ****************************  data transformations  ****************************


DATA_prepare.ID = (1:length(DATA_prepare.record))';
n = length(DATA_prepare.ID);

% Transforming data to long format

% Change ScenarioC_XHabAmt3 to categorical (for consistency) 
for cs = 1:12
    eval(['DATA_prepare.ScenarioC_' num2str(cs) 'HabAmt3 = categorical(DATA_prepare.ScenarioC_' num2str(cs) 'HabAmt3);']);    
end

% Create auxiliary matrix with attributes
Recov_aux = [];
HabAmt_aux = [];
Sites_aux = [];
Cost_aux = [];
habitat_presented_aux = [];
block_aux = [];
ordershown_aux = [];
timer_aux = [];

for ct = 1 : 12
    eval(['Recov_aux = [Recov_aux, DATA_prepare.ScenarioA_' num2str(ct) 'Recov1, DATA_prepare.ScenarioB_' num2str(ct) 'Recov2, DATA_prepare.ScenarioC_' num2str(ct) 'Recov3];']);
    eval(['HabAmt_aux = [HabAmt_aux, DATA_prepare.ScenarioA_' num2str(ct) 'HabAmt1, DATA_prepare.ScenarioB_' num2str(ct) 'HabAmt2, DATA_prepare.ScenarioC_' num2str(ct) 'HabAmt3];']);
    eval(['Sites_aux = [Sites_aux, DATA_prepare.ScenarioA_' num2str(ct) 'Sites1, DATA_prepare.ScenarioB_' num2str(ct) 'Sites2, DATA_prepare.ScenarioC_' num2str(ct) 'Sites3];']);
    eval(['Cost_aux = [Cost_aux, DATA_prepare.ScenarioA_' num2str(ct) 'Cost, DATA_prepare.ScenarioB_' num2str(ct) 'Cost2, DATA_prepare.ScenarioC_' num2str(ct) 'Cost3];']);
    eval(['habitat_presented_aux = [habitat_presented_aux, DATA_prepare.LoopInfo_' num2str(ct) 'habitatpresented, DATA_prepare.LoopInfo_' num2str(ct) 'habitatpresented, DATA_prepare.LoopInfo_' num2str(ct) 'habitatpresented];']);
    eval(['block_aux = [block_aux, DATA_prepare.LoopInfo_' num2str(ct) 'block, DATA_prepare.LoopInfo_' num2str(ct) 'block, DATA_prepare.LoopInfo_' num2str(ct) 'block];']);
    eval(['ordershown_aux = [ordershown_aux, DATA_prepare.LoopInfo_' num2str(ct) 'ordershown, DATA_prepare.LoopInfo_' num2str(ct) 'ordershown, DATA_prepare.LoopInfo_' num2str(ct) 'ordershown];']);    
    eval(['timer_aux = [timer_aux, DATA_prepare.Q17_Timer_' num2str(ct) ', DATA_prepare.Q17_Timer_' num2str(ct) ', DATA_prepare.Q17_Timer_' num2str(ct) '];']);    
end

% Decoding categorical variables

% Recov:
% 0 - No change
% 1 - Minimal  -> Low       wildlife presence
% 2 - Low      -> Moderate  wilflife presence
% 3 - Moderate -> Full      wildlife presense
Recov_aux = string(Recov_aux);
Recov_aux(Recov_aux=="No change")="0";
Recov_aux(Recov_aux=="Minimal wildlife presence <b>→ Low wildlife presence</b>")="1";
Recov_aux(Recov_aux=="Low wildlife presence <b>→ Moderate wildlife presence</b>")="2";
Recov_aux(Recov_aux=="Moderate wildlife presence <b>→ Full wildlife presence</b>")="3";
Recov_aux = double(Recov_aux);

% Sites:
% 0 - N/A
% 1 - Small sites
% 2 - Medium sites
% 3 - Large sites
Sites_aux = string(Sites_aux);
Sites_aux(Sites_aux=="N/A")="0";
Sites_aux(Sites_aux=="<b>Small sites</b>")="1";
Sites_aux(Sites_aux=="<b>Medium sites</b>")="2";
Sites_aux(Sites_aux=="<b>Large sites</b>")="3";
Sites_aux = double(Sites_aux);

Cost_aux = string(Cost_aux);
Cost_aux(Cost_aux=="<b>No additional cost</b>")="0";
Cost_aux(Cost_aux=="<b>£5 per year</b><br />(£0.42 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="5";
Cost_aux(Cost_aux=="<b>£10 per year</b><br />(£0.83 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="10";
Cost_aux(Cost_aux=="<b>£25 per year</b><br />(£2.08 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="25";
Cost_aux(Cost_aux=="<b>£50 per year</b><br />(£4.17 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="50";
Cost_aux(Cost_aux=="<b>£75 per year</b><br />(£6.25 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="75";
Cost_aux(Cost_aux=="<b>£100 per year</b><br />(£8.33 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="100";
Cost_aux(Cost_aux=="<b>£150 per year</b><br />(£12.50 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="150";
Cost_aux(Cost_aux=="<b>£250 per year</b><br />(£20.83 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="250";
Cost_aux(Cost_aux=="<b>£500 per year</b><br />(£41.67 per month)<br /><br /><span class= greenText ><i>Amount from 2023 to 2042</i></span>")="500";
Cost_aux = double(Cost_aux);   

HabAmt_ha = string(HabAmt_aux);
HabAmt_ha(contains(HabAmt_ha,"190,000 Hectares")) = "190000";
HabAmt_ha(contains(HabAmt_ha,"150,000 Hectares")) = "150000";
HabAmt_ha(contains(HabAmt_ha,"140,000 Hectares")) = "140000";
HabAmt_ha(contains(HabAmt_ha,"130,000 Hectares")) = "130000";
HabAmt_ha(contains(HabAmt_ha,"120,000 Hectares")) = "120000";
HabAmt_ha(contains(HabAmt_ha,"110,000 Hectares")) = "110000";
HabAmt_ha(contains(HabAmt_ha,"100,000 Hectares")) = "100000";
HabAmt_ha(contains(HabAmt_ha,"88,000 Hectares")) = "88000";
HabAmt_ha(contains(HabAmt_ha,"86,000 Hectares")) = "86000";
HabAmt_ha(contains(HabAmt_ha,"83,000 Hectares")) = "83000";
HabAmt_ha(contains(HabAmt_ha,"75,000 Hectares")) = "75000";
HabAmt_ha(contains(HabAmt_ha,"68,000 Hectares")) = "68000";
HabAmt_ha(contains(HabAmt_ha,"65,000 Hectares")) = "65000";
HabAmt_ha(contains(HabAmt_ha,"59,000 Hectares")) = "59000";
HabAmt_ha(contains(HabAmt_ha,"55,000 Hectares")) = "55000";
HabAmt_ha(contains(HabAmt_ha,"50,000 Hectares")) = "50000";
HabAmt_ha(contains(HabAmt_ha,"43,000 Hectares")) = "43000";
HabAmt_ha(contains(HabAmt_ha,"38,000 Hectares")) = "38000";
HabAmt_ha(contains(HabAmt_ha,"34,000 Hectares")) = "34000";
HabAmt_ha(contains(HabAmt_ha,"29,000 Hectares")) = "29000";
HabAmt_ha(contains(HabAmt_ha,"28,000 Hectares")) = "28000";
HabAmt_ha(contains(HabAmt_ha,"25,000 Hectares")) = "25000";
HabAmt_ha(contains(HabAmt_ha,"22,000 Hectares")) = "22000";
HabAmt_ha(contains(HabAmt_ha,"19,000 Hectares")) = "19000";
HabAmt_ha(contains(HabAmt_ha,"17,000 Hectares")) = "17000";
HabAmt_ha(contains(HabAmt_ha,"15,000 Hectares")) = "15000";
HabAmt_ha(contains(HabAmt_ha,"14,000 Hectares")) = "14000";
HabAmt_ha(contains(HabAmt_ha,"13,000 Hectares")) = "13000";
HabAmt_ha(contains(HabAmt_ha,"12,000 Hectares")) = "12000";
HabAmt_ha(contains(HabAmt_ha,"11,000 Hectares")) = "11000";
HabAmt_ha(contains(HabAmt_ha,"10,000 Hectares")) = "10000";
HabAmt_ha(contains(HabAmt_ha,"9,000 Hectares")) = "9000";
HabAmt_ha(contains(HabAmt_ha,"8,600 Hectares")) = "8600";
HabAmt_ha(contains(HabAmt_ha,"7,500 Hectares")) = "7500";
HabAmt_ha(contains(HabAmt_ha,"7,000 Hectares")) = "7000";
HabAmt_ha(contains(HabAmt_ha,"6,000 Hectares")) = "6000";
HabAmt_ha(contains(HabAmt_ha,"5,900 Hectares")) = "5900";
HabAmt_ha(contains(HabAmt_ha,"5,800 Hectares")) = "5800";
HabAmt_ha(contains(HabAmt_ha,"5,000 Hectares")) = "5000";
HabAmt_ha(contains(HabAmt_ha,"4,000 Hectares")) = "4000";
HabAmt_ha(contains(HabAmt_ha,"2,900 Hectares")) = "2900";
HabAmt_ha(contains(HabAmt_ha,"2,500 Hectares")) = "2500";
HabAmt_ha(contains(HabAmt_ha,"2,000 Hectares")) = "2000";
HabAmt_ha(contains(HabAmt_ha,"1,500 Hectares")) = "1500";
HabAmt_ha(contains(HabAmt_ha,"1,200 Hectares")) = "1200";
HabAmt_ha(contains(HabAmt_ha,"1,000 Hectares")) = "1000";
HabAmt_ha(contains(HabAmt_ha,"700 Hectares")) = "700";
HabAmt_ha(contains(HabAmt_ha,"600 Hectares")) = "600";
HabAmt_ha(contains(HabAmt_ha,"500 Hectares")) = "500";
HabAmt_ha = double(HabAmt_ha);

%HabAmt_perc (2.5; 5; 10; 12.5; 15; 20; 25; 37.5; 50)
HabAmt_perc = string(HabAmt_aux);
HabAmt_perc(contains(HabAmt_perc,"About 2.5%")) = "2.5";
HabAmt_perc(contains(HabAmt_perc,"About 5.0%")) = "5";
HabAmt_perc(contains(HabAmt_perc,"About 10.0%")) = "10";
HabAmt_perc(contains(HabAmt_perc,"About 12.5%")) = "12.5";
HabAmt_perc(contains(HabAmt_perc,"About 15.0%")) = "15";
HabAmt_perc(contains(HabAmt_perc,"About 20.0%")) = "20";
HabAmt_perc(contains(HabAmt_perc,"About 25.0%")) = "25";
HabAmt_perc(contains(HabAmt_perc,"About 37.5%")) = "37.5";
HabAmt_perc(contains(HabAmt_perc,"About 50.0%")) = "50";
HabAmt_perc = double(HabAmt_perc);
HabAmt_perc = HabAmt_perc/100;

% habitat_presented
% 1 - Arable land 
% 2 - Blanket bog 
% 3 - Heathland 
% 4 - Lowland fens 
% 5 - Lowland hay meadows 
% 6 - Native mixed deciduous woodland 
% 7 - Semi-natural grassland 
% 8 - Upland oakwood 
% 9 - Wood pasture and parkland 
% 10 - Rivers 
% 11 - Coastal sand dunes 
habitat_presented_aux = string(habitat_presented_aux);
habitat_presented_aux(contains(habitat_presented_aux,"Arable land")) = "1";
habitat_presented_aux(contains(habitat_presented_aux,"Blanket bog")) = "2";
habitat_presented_aux(contains(habitat_presented_aux,"Heathland")) = "3";
habitat_presented_aux(contains(habitat_presented_aux,"Lowland fens")) = "4";
habitat_presented_aux(contains(habitat_presented_aux,"Lowland hay meadows")) = "5";
habitat_presented_aux(contains(habitat_presented_aux,"Native mixed deciduous woodland")) = "6";
habitat_presented_aux(contains(habitat_presented_aux,"Semi-natural grassland")) = "7";
habitat_presented_aux(contains(habitat_presented_aux,"Upland oakwood")) = "8";
habitat_presented_aux(contains(habitat_presented_aux,"Wood pasture and parkland")) = "9";
habitat_presented_aux(contains(habitat_presented_aux,"Rivers")) = "10";
habitat_presented_aux(contains(habitat_presented_aux,"Coastal sand dunes")) = "11";
habitat_presented_aux = double(habitat_presented_aux);

% dstats(habitat_presented_aux,1)

% Choice
Choice_aux = [];

for ct = 1 : 12
    eval(['Choice_aux = [Choice_aux, DATA_prepare.Q17_' num2str(ct) ', DATA_prepare.Q17_' num2str(ct) ', DATA_prepare.Q17_' num2str(ct) '];']);
end

% DCE data
DATA.ID = ReplicateRows((1:n)',NAlt*NCT);
DATA.NAlt = reshape(repmat([1:3]',n*NCT,1)',1,[])';
DATA.CS = reshape(repmat([1:12;1:12;1:12]',n,1)',1,[])';

DATA.Choice_aux = reshape(Choice_aux',[],1);
DATA.Choice = double(DATA.Choice_aux == DATA.NAlt);
DATA.Recov = reshape(Recov_aux',[],1);
DATA.Sites = reshape(Sites_aux',[],1);
DATA.Cost = reshape(Cost_aux',[],1);
DATA.HabAmt_ha = reshape(HabAmt_ha',[],1);
DATA.HabAmt_perc = reshape(HabAmt_perc',[],1);
DATA.habitat_presented = reshape(habitat_presented_aux',[],1);
DATA.block = reshape(block_aux',[],1);
DATA.order_shown = reshape(ordershown_aux',[],1);
DATA.timer = reshape(timer_aux',[],1);

testDCE = [DATA.ID, DATA.NAlt, DATA.CS, DATA.Choice, DATA.Recov, DATA.Sites, DATA.Cost, DATA.HabAmt_ha, DATA.HabAmt_perc, DATA.habitat_presented, DATA.block, DATA.order_shown, DATA.timer];
testDCE = array2table(testDCE, 'VariableNames', {'Id', 'Alt','CS', 'Choice', 'Recov', 'Sites', 'Cost', 'HabAmt_ha', 'HabAmt_perc', 'habitat_presented', 'block', 'order_shown', 'timer'});

clear block_aux Choice_aux Cost_aux HabAmt_aux HabAmt_ha HabAmt_perc habitat_presented_aux ordershown_aux Recov_aux Sites_aux timer_aux


%% ****************************  other variables  ****************************

% [record]: Record number		Open numeric response	
DATA.record = ReplicateRows(DATA_prepare.record,NCT*NAlt,1);

% [uuid]: Participant identifier		Open text response	
DATA.uuid = ReplicateRows(DATA_prepare.uuid,NCT*NAlt,1);

%[date]: Completion time and date		Open text response	
DATA.date = ReplicateRows(DATA_prepare.date1,NCT*NAlt,1);

% [status]: Participant status		Values: 1-4		
%	1	Terminated
%	2	Overquota
%	3	Qualified
%	4	Partial
DATA.status = ReplicateRows(DATA_prepare.status,NCT*NAlt,1);
    
%[dSurveyVersion]: Survey Version		Values: 1-6		
% 	1	Focus Group
% 	2	Soft launch (250)
% 	3	Full Launch 1 (1000)
% 	4	Full Launch 2 (remaining)
% 	5	TBC
% 	6	TBC
DATA.dSurveyVersion = ReplicateRows(DATA_prepare.dSurveyVersion,NCT*NAlt,1);

%[dSurveyType]: Survey Type		Values: 1-3		
% 	1	Focus Group
% 	2	Main
% 	3	Focus to Main
DATA.dSurveyType = ReplicateRows(DATA_prepare.dSurveyType,NCT*NAlt,1);
    
%FocusGroupURL: the version/set from the focus group url **SKIP IF EMPTY**		Open text response		
% 	[FocusGroupURLr1]	ID
% 	[FocusGroupURLr2]	Version
% 	[FocusGroupURLr3]	Set
% 	[FocusGroupURLr4]	Habitat
DATA.FocusGroupURLr1 = ReplicateRows(DATA_prepare.FocusGroupURLr1,NCT*NAlt,1);
DATA.FocusGroupURLr2 = ReplicateRows(DATA_prepare.FocusGroupURLr2,NCT*NAlt,1);
DATA.FocusGroupURLr3 = ReplicateRows(DATA_prepare.FocusGroupURLr3,NCT*NAlt,1);
DATA.FocusGroupURLr4 = ReplicateRows(DATA_prepare.FocusGroupURLr4,NCT*NAlt,1);
    
%[HabitatDesignVersion]: YOU CAN PICK THE VERSION HERE (1-165) - ENTER A SINGLE NUMERIC VALUEHabitat Design Version	 Values: 1-165	
DATA.HabitatDesignVersion = ReplicateRows(DATA_prepare.HabitatDesignVersion,NCT*NAlt,1);

%[hHabitat1; hHabitat2; hHabitat3]: the habitats		Values: 1-11		
% 	1	Wood pasture and parkland
% 	2	Native mixed deciduous woodland
% 	3	Upland oakwood
% 	4	Arable land
% 	5	Lowland hay meadows
% 	6	Semi-natural grassland
% 	7	Heathland
% 	8	Lowland fens
% 	9	Blanket bog
% 	10	Rivers
% 	11	Coastal sand dunes
DATA.hHabitat1 = ReplicateRows(DATA_prepare.hHabitat1,NCT*NAlt,1);
DATA.hHabitat2 = ReplicateRows(DATA_prepare.hHabitat2,NCT*NAlt,1);
DATA.hHabitat3 = ReplicateRows(DATA_prepare.hHabitat3,NCT*NAlt,1);

%COMB: the habitats combination		Open text response		
% 	[COMBr1]	Comb
% 	[COMBr2]	Comb text
DATA.COMBr1 = ReplicateRows(DATA_prepare.COMBr1,NCT*NAlt,1);
DATA.COMBr2 = ReplicateRows(DATA_prepare.COMBr2,NCT*NAlt,1);

%[Q1]: First, please answer some questions about yourself and your household. This will help us check that we have surveyed a wide spread of people across England.Please select the region of England you live in.		Values: 1-10		
% 	1	North East
% 	2	North West
% 	3	Yorkshire and the Humber
% 	4	East Midlands
% 	5	West Midlands
% 	6	East of England
% 	7	London
% 	8	South East
% 	9	South West
% 	10	None of these
DATA.Q1 = ReplicateRows(DATA_prepare.Q1,NCT*NAlt,1);

%[Q2]: Are you solely or jointly responsible for paying household bills in your household including, for example, council tax?		Values: 1-4		
% 	1	Solely responsible
% 	2	Jointly responsible
% 	3	Not responsible
% 	4	Don't know
DATA.Q2 = ReplicateRows(DATA_prepare.Q2,NCT*NAlt,1);
% 
% [Q3]: Please can you indicate your age.		Values: 1-8		
% 	1	Under 16
% 	2	16-17
% 	3	18-24
% 	4	25-34
% 	5	35-44
% 	6	45-54
% 	7	55-64
% 	8	65+
DATA.Q3 = ReplicateRows(DATA_prepare.Q3,NCT*NAlt,1);

% [Q4]: Please indicate your gender.		Values: 1-4		
% 	1	Male
% 	2	Female
% 	3	Prefer to identify another way
% 	4	Prefer not to say
DATA.Q4 = ReplicateRows(DATA_prepare.Q4,NCT*NAlt,1);

% [Q5]: Are you the main income earner in your household?		Values: 1-4		
% 	1	Yes
% 	2	No
% 	3	Joint – my partner and I earn similar incomes
% 	4	No income earners
DATA.Q5 = ReplicateRows(DATA_prepare.Q5,NCT*NAlt,1);

% [Q6]: Main income earner's occupation (if main income earner is retired, please select occupation before retirement). Rollover each occupation type for more information or review the image below. Please click on the image above to enlarge it.		Values: 1-6		
% 	1	Higher managerial, administrative or professionalEstablished doctor, solicitor, board director in a large organisation (200+ employees), top level civil servant/public service employee
% 	2	Intermediate managerial, administrative or professionalNewly qualified (under 3 years) doctor, solicitor, board director of small organisation, middle manager in large organisation, principal officer in civil service/local government
% 	3	Supervisory or clerical and junior managerial, administrative or professionalOffice worker, student doctor, foreman with 25+ employees, salesperson
% 	4	Skilled manual workerSkilled bricklayer, carpenter, plumber, painter, bus/ambulance driver, HGV driver, pub/bar worker
% 	5	Semi or unskilled manual workerManual workers, all apprentices to skilled trades, caretaker, park keeper, non-HGV driver, shop assistant
% 	6	Casual worker, dependent on state pension only, or dependent on state welfareCasual worker (not in permanent employment), housewife/homemaker, retired and living on state pension, unemployed or not working due to long-term sickness, full-time carer of other household member
DATA.Q6 = ReplicateRows(DATA_prepare.Q6,NCT*NAlt,1);

% [hQ6]: Main income earner's occupation (if main income earner is retired, please select occupation before retirement).		Values: 1-6		
% 	1	A
% 	2	B
% 	3	C1
% 	4	C2
% 	5	D
% 	6	E
DATA.hQ6 = ReplicateRows(DATA_prepare.hQ6,NCT*NAlt,1);

% [Q7]: Which of the following best describes the place where you live?		Values: 1-4		
% 	1	City or town centre (i.e. close to main retail and commercial areas)
% 	2	Suburbs or housing development on edge of town (i.e. mostly residential area)
% 	3	Mainly rural area (i.e. countryside, village, or market town)
% 	4	Other (please specify)
DATA.Q7 = ReplicateRows(DATA_prepare.Q7,NCT*NAlt,1);

%[Q7r4oe]: Which of the following best describes the place where you live? - Other (please specify)Open text response
DATA.Q7r4oe = ReplicateRows(DATA_prepare.Q7r4oe,NCT*NAlt,1);

% Q8: Please look at these different aspects of natural areas. How important is each to you?		Values: 1-4		
% 	1	Very important
% 	2	Somewhat important
% 	3	Not important at all
% 	4	Don't know
% 	[Q8r1]	The animals and plants within the area being in a natural or "wild" state
% 	[Q8r2]	Seeing "working" landscapes like agricultural fields (crops, cows, sheep, etc.)
% 	[Q8r3]	"Pristine" and unspoilt views of the natural landscape (e.g. roads, powerlines, buildings, quarries, or other human developments are not present)
% 	[Q8r4]	A wide variety of different trees, plants, and fungi being present
% 	[Q8r5]	The presence of wild animals like birds, mammals, and reptiles
DATA.Q8r1 = ReplicateRows(DATA_prepare.Q8r1,NCT*NAlt,1);
DATA.Q8r2 = ReplicateRows(DATA_prepare.Q8r2,NCT*NAlt,1);
DATA.Q8r3 = ReplicateRows(DATA_prepare.Q8r3,NCT*NAlt,1);
DATA.Q8r4 = ReplicateRows(DATA_prepare.Q8r4,NCT*NAlt,1);
DATA.Q8r5 = ReplicateRows(DATA_prepare.Q8r5,NCT*NAlt,1);

% [Q9]: The amount and diversity of wild species of plants and animals in nature form the basis for a healthy natural environment. Which of these statements do you think applies to the wild species present where you live?		Values: 1-4		
% 	1	I think there is a wide range of wild species and a healthy natural environment in my local area relative to the rest of England
% 	2	I think the state of wild species and the natural environment in my local area is about average relative to the rest of England
% 	3	I think there is a limited range of wild species and a degraded natural environment in my local area relative to the rest of England
% 	4	I am not very aware of the presence of wild species and the state of the natural environment in my local area
DATA.Q9 = ReplicateRows(DATA_prepare.Q9,NCT*NAlt,1);

% [Q10]: The most recent “State of Nature Report” shows that since the 1970s (when reliable records began) there has been a steady decline in the number of wild animal, plant, and fungi species in the country as well as a decline in the number of places where these species live. Please click on the image above to enlarge it. Prior to this survey, how aware were you of the decline in nature and wildlife in England?		Values: 1-3		
% 	1	Very aware
% 	2	Somewhat aware
% 	3	Not aware
DATA.Q10 = ReplicateRows(DATA_prepare.Q10,NCT*NAlt,1);

% [Q11]: The decline in nature and wildlife has been caused by many factors. This includes impacts from agriculture, loss of green spaces in towns and cities, and changing climate. Please click on the image above to enlarge it. Which of these statements do you agree with the most?		Values: 1-3		
% 	1	Taking action to restore nature and improve areas for wildlife should be one of the highest priorities for Government
% 	2	Loss of nature and wildlife concerns me, but there are more important day-to-day problems to address in the country
% 	3	Loss of nature and wildlife does not matter to me
DATA.Q11 = ReplicateRows(DATA_prepare.Q11,NCT*NAlt,1);

% [Q12]: Restoring nature and wildlife will require action from different parts of society, including the Government, farming, industry, utilities (such as water companies), Local Authorities, landowners, conservation organisations, community groups, and individuals. Please click on the image above to enlarge it. How familiar are you with these types of actions?		Values: 1-3		
% 	1	I am familiar with all of these types of actions
% 	2	I am familiar with some of these actions; others are new to me
% 	3	I am not familiar with these actions; all are new to me
DATA.Q12 = ReplicateRows(DATA_prepare.Q12,NCT*NAlt,1);

% Q13: To what extent do you agree or disagree with these statements?		Values: 1-5		
% 	1	Strongly agree
% 	2	Agree
% 	3	Neither agree nor disagree
% 	4	Disagree
% 	5	Strongly disagree
% 	[Q13r1]	I would prefer more land be dedicated to creating or maintaining natural habitats that can support increased wild species presence
% 	[Q13r2]	I would be happy to change certain aspects of my lifestyle if it meant that more land could be returned to natural habitats that can support increased wild species presence
% 	[Q13r3]	It is society's responsibility to take actions to help wild species recovery
% 	[Q13r4]	People's needs should take priority over nature and wildlife when they come into conflict
DATA.Q13r1 = ReplicateRows(DATA_prepare.Q13r1,NCT*NAlt,1);
DATA.Q13r2 = ReplicateRows(DATA_prepare.Q13r2,NCT*NAlt,1);
DATA.Q13r3 = ReplicateRows(DATA_prepare.Q13r3,NCT*NAlt,1);
DATA.Q13r4 = ReplicateRows(DATA_prepare.Q13r4,NCT*NAlt,1);

% Q14_Q15_Q16: How familiar are you with this habitat?		Values: 1-4		
% 	1	Very familiar
% 	2	Somewhat familiar
% 	3	Not familiar at all
% 	4	Don't know
% 	[Q14_Q15_Q16Hab1]	Wood pasture and parkland
% 	[Q14_Q15_Q16Hab2]	Native mixed deciduous woodland
% 	[Q14_Q15_Q16Hab3]	Upland oakwood
% 	[Q14_Q15_Q16Hab4]	Arable land
% 	[Q14_Q15_Q16Hab5]	Lowland hay meadows
% 	[Q14_Q15_Q16Hab6]	Semi-natural grassland
% 	[Q14_Q15_Q16Hab7]	Heathland
% 	[Q14_Q15_Q16Hab8]	Lowland fens
% 	[Q14_Q15_Q16Hab9]	Blanket bog
% 	[Q14_Q15_Q16Hab10]	Rivers
% 	[Q14_Q15_Q16Hab11]	Coastal sand dunes
DATA.Q14_Q15_Q16Hab1 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab1,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab2 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab2,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab3 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab3,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab4 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab4,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab5 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab5,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab6 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab6,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab7 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab7,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab8 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab8,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab9 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab9,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab10 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab10,NCT*NAlt,1);
DATA.Q14_Q15_Q16Hab11 = ReplicateRows(DATA_prepare.Q14_Q15_Q16Hab11,NCT*NAlt,1);

%[Q17_Version]: HIDDEN WHEN LIVE: YOU CAN PICK THE VERSION HERE - ENTER A SINGLE NUMERIC VALUE between 1-6Block - DCM Version	Values: 1-6	
DATA.Q17_Version = ReplicateRows(DATA_prepare.Q17_Version,NCT*NAlt,1);

% ScenarioAnswerCount: The number of times each scenario was selected	Values: 0-99	
% 	[ScenarioAnswerCountA]
% 	[ScenarioAnswerCountB]
% 	[ScenarioAnswerCountC]
DATA.ScenarioAnswerCountA = ReplicateRows(DATA_prepare.ScenarioAnswerCountA,NCT*NAlt,1);
DATA.ScenarioAnswerCountB = ReplicateRows(DATA_prepare.ScenarioAnswerCountB,NCT*NAlt,1);
DATA.ScenarioAnswerCountC = ReplicateRows(DATA_prepare.ScenarioAnswerCountC,NCT*NAlt,1);

% [Q18]: Considering the information and instructions provided, how easy or difficult was it to answer the choice questions?		Values: 1-5		
% 	1	Very easy
% 	2	Fairly easy
% 	3	Neither easy nor difficult
% 	4	Fairly difficult
% 	5	Very difficult
DATA.Q18 = ReplicateRows(DATA_prepare.Q18,NCT*NAlt,1);

% Q19: Were the choice questions difficult because…?		Values: 0-1		
% 	0	Unchecked
% 	1	Checked
% 	[Q19r1]	It was hard to decide which scenario I preferred
% 	[Q19r2]	Not enough information was provided about the alternative scenarios to make a choice
% 	[Q19r3]	The instructions for the choice questions were not clear
% 	[Q19r4]	I was confused / didn’t understand the choices
% 	[Q19r5]	Other (please specify)
DATA.Q19r1 = ReplicateRows(DATA_prepare.Q19r1,NCT*NAlt,1);
DATA.Q19r2 = ReplicateRows(DATA_prepare.Q19r2,NCT*NAlt,1);
DATA.Q19r3 = ReplicateRows(DATA_prepare.Q19r3,NCT*NAlt,1);
DATA.Q19r4 = ReplicateRows(DATA_prepare.Q19r4,NCT*NAlt,1);
DATA.Q19r5 = ReplicateRows(DATA_prepare.Q19r5,NCT*NAlt,1);

%[Q19r5oe]: Were the choice questions difficult because…? - Other (please specify) Open text response
DATA.Q19r5oe = ReplicateRows(DATA_prepare.Q19r5oe,NCT*NAlt,1);

% Q20: How important was each aspect of the scenarios to the choices you made?		Values: 1-5		
% 	1	Very important (considered it in all choices)
% 	2	Quite important (considered it in most choices)
% 	3	Not very important (ignored it in most choices)
% 	4	Not at all important (ignored it in all choices)
% 	5	Don't know
% 	[Q20r1]	The particular habitats for plant, animal and fungi species I was shown
% 	[Q20r2]	The change in the wild species presence
% 	[Q20r3]	The amount of habitat improved
% 	[Q20r4]	The type of sites targeted by recovery actions
% 	[Q20r5]	Increase in household expenditure
DATA.Q20r1 = ReplicateRows(DATA_prepare.Q20r1,NCT*NAlt,1);
DATA.Q20r2 = ReplicateRows(DATA_prepare.Q20r2,NCT*NAlt,1);
DATA.Q20r3 = ReplicateRows(DATA_prepare.Q20r3,NCT*NAlt,1);
DATA.Q20r4 = ReplicateRows(DATA_prepare.Q20r4,NCT*NAlt,1);
DATA.Q20r5 = ReplicateRows(DATA_prepare.Q20r5,NCT*NAlt,1);

% [Q21]: Thinking about the choices you made, which of the statements below best describes how you chose between the different options?		Values: 1-7		
% 	1	I chose options with the least cost to my household
% 	2	I chose options that were best for wild species recovery relative to the cost
% 	3	I chose options that were best for wild species recovery irrespective of cost
% 	4	I chose the options that would affect or are most likely to affect my household directly
% 	5	I chose options with improvements I thought were necessary
% 	6	I chose options that I thought would benefit people I know
% 	7	Other (please specify)
DATA.Q21 = ReplicateRows(DATA_prepare.Q21,NCT*NAlt,1);

%[Q21r7oe]: Thinking about the choices you made, which of the statements below best describes how you chose between the different options? - Other (please specify)	Open text response	
DATA.Q21r7oe = ReplicateRows(DATA_prepare.Q21r7oe,NCT*NAlt,1);

% Q22: When making your choices, were you thinking of what species recovery and improving nature and wildlife would mean...		Values: 1-4		
% 	1	Yes - I considered this when making my choices
% 	2	Somewhat - I sometimes considered this when making my choices
% 	3	No - I did not consider this when making my choices
% 	4	Don't know
% 	[Q22r1]	To you
% 	[Q22r2]	To your household and family
% 	[Q22r3]	To wider society
% 	[Q22r4]	To future generations
% 	[Q22r5]	For nature and wildlife
DATA.Q22r1 = ReplicateRows(DATA_prepare.Q22r1,NCT*NAlt,1);
DATA.Q22r2 = ReplicateRows(DATA_prepare.Q22r2,NCT*NAlt,1);
DATA.Q22r3 = ReplicateRows(DATA_prepare.Q22r3,NCT*NAlt,1);
DATA.Q22r4 = ReplicateRows(DATA_prepare.Q22r4,NCT*NAlt,1);
DATA.Q22r5 = ReplicateRows(DATA_prepare.Q22r5,NCT*NAlt,1);

% [Q23]: You selected "no change - no increase in species recovery actions" in most of your choices. What was the main reason for this?		Values: 1-12		
% 	1	I object to the proposed actions for species recovery
% 	2	I object to paying higher bills
% 	3	The Local Authority (council) should pay for this
% 	4	The Government should pay for this
% 	5	Farmers and landowners should pay for this
% 	6	I do not believe the species recovery actions would actually happen
% 	7	I would like the species recovery actions, but I cannot afford to pay for them
% 	8	Species recovery is not important to me
% 	9	I think the current state of nature and wildlife is already good enough
% 	10	The information provided was not clear enough for me to make a choice
% 	11	Humans have a moral duty to protect nature and wildlife – we should not be concerned about the costs
% 	12	Other (please specify)
DATA.Q23 = ReplicateRows(DATA_prepare.Q23,NCT*NAlt,1);

%[Q23r12oe]: You selected "no change - no increase in species recovery actions" in most of your choices. What was the main reason for this? - Other (please specify)	Open text response	
DATA.Q23r12oe = ReplicateRows(DATA_prepare.Q23r12oe,NCT*NAlt,1);

% [Q24ac1]: You indicated that - at least in some cases - you would be willing to pay for species recovery actions to improve nature and wildlife. Please could you indicate the main reasons why from the statements below? Please select the most important reason from the list and any others that apply.		Values: 1-8		
% 	1	Species recovery will improve the areas I like to visit
% 	2	I like to know that nature and wildlife are present and healthy, even if I never visit the areas or see them
% 	3	Species recovery will benefit other people who visit the sites
% 	4	Species recovery will ensure that nature and wildlife can be enjoyed by future generations
% 	5	Species recovery will benefit businesses and the economy
% 	6	Nature and wildlife have a right to exist
% 	7	Humans have a moral duty to protect nature and wildlife, and correct the damage they have caused
% 	8	Other reason
DATA.Q24ac1 = ReplicateRows(DATA_prepare.Q24ac1,NCT*NAlt,1);

% Q24b: You indicated that - at least in some cases - you would be willing to pay for species recovery actions to improve nature and wildlife. Please could you indicate the main reasons why from the statements below? Please select the most important reason from the list and any others that apply.		Values: 0-1		
% 	0	Unchecked
% 	1	Checked
% 	[Q24br1c2]	Species recovery will improve the areas I like to visit
% 	[Q24br2c2]	I like to know that nature and wildlife are present and healthy, even if I never visit the areas or see them
% 	[Q24br3c2]	Species recovery will benefit other people who visit the sites
% 	[Q24br4c2]	Species recovery will ensure that nature and wildlife can be enjoyed by future generations
% 	[Q24br5c2]	Species recovery will benefit businesses and the economy
% 	[Q24br6c2]	Nature and wildlife have a right to exist
% 	[Q24br7c2]	Humans have a moral duty to protect nature and wildlife, and correct the damage they have caused
% 	[Q24br8c2]	Other reason
DATA.Q24br1c2 = ReplicateRows(DATA_prepare.Q24br1c2,NCT*NAlt,1);
DATA.Q24br2c2 = ReplicateRows(DATA_prepare.Q24br2c2,NCT*NAlt,1);
DATA.Q24br3c2 = ReplicateRows(DATA_prepare.Q24br3c2,NCT*NAlt,1);
DATA.Q24br4c2 = ReplicateRows(DATA_prepare.Q24br4c2,NCT*NAlt,1);
DATA.Q24br5c2 = ReplicateRows(DATA_prepare.Q24br5c2,NCT*NAlt,1);
DATA.Q24br6c2 = ReplicateRows(DATA_prepare.Q24br6c2,NCT*NAlt,1);
DATA.Q24br7c2 = ReplicateRows(DATA_prepare.Q24br7c2,NCT*NAlt,1);
DATA.Q24br8c2 = ReplicateRows(DATA_prepare.Q24br8c2,NCT*NAlt,1);

% Q25: There are different ways that nature and wild species recovery can be achieved. Please could you rank the actions below in terms of what you think should be highest priority.		Values: 1-6		
% 	1	Highest priority
% 	2	2nd priority
% 	3	3rd priority
% 	4	4th priority
% 	5	5th priority
% 	6	Least priority
% 	[Q25r1]	Better agricultural practices such as "wildlife friendly farming"
% 	[Q25r2]	"Making space for nature" in towns and cities
% 	[Q25r3]	Restoring natural habitats for wildlife, or improving degraded habitats
% 	[Q25r4]	Reducing air and water pollution and contaminated land
% 	[Q25r5]	"Reintroducing" plants and animals that have been lost
% 	[Q25r6]	Better enforcement of laws that protect plants and animals from harm
DATA.Q25r1 = ReplicateRows(DATA_prepare.Q25r1,NCT*NAlt,1);
DATA.Q25r2 = ReplicateRows(DATA_prepare.Q25r2,NCT*NAlt,1);
DATA.Q25r3 = ReplicateRows(DATA_prepare.Q25r3,NCT*NAlt,1);
DATA.Q25r4 = ReplicateRows(DATA_prepare.Q25r4,NCT*NAlt,1);
DATA.Q25r5 = ReplicateRows(DATA_prepare.Q25r5,NCT*NAlt,1);
DATA.Q25r6 = ReplicateRows(DATA_prepare.Q25r6,NCT*NAlt,1);

% Q26: How certain are you that your responses to this survey will…		Values: 1-5		
% 	1	Very certain
% 	2	Fairly certain
% 	3	Not very certain
% 	4	Not certain at all
% 	5	Don't know
% 	[Q26r1]	Influence the actions that are taken to protect and improve nature and wildlife
% 	[Q26r2]	Result in changes to your household expenditure that pay for wild species recovery actions
DATA.Q26r1 = ReplicateRows(DATA_prepare.Q26r1,NCT*NAlt,1);
DATA.Q26r2 = ReplicateRows(DATA_prepare.Q26r2,NCT*NAlt,1);

% [Q27]: Which of the following best describes your household?		Values: 1-10		
% 	1	Single working age adult
% 	2	Single retired age adult
% 	3	Two adults of working age
% 	4	Two adults of retired age
% 	5	More than two adults, no children (below 18 years old)
% 	6	Single parent family with fewer than 3 children (below 18 years old)
% 	7	Two parent family with fewer than 3 children (below 18 years old)
% 	8	Family with 3 or more children (below 18 years old)
% 	9	Other (please specify)
% 	10	Prefer not to say
DATA.Q27 = ReplicateRows(DATA_prepare.Q27,NCT*NAlt,1);

%[Q27r9oe]: Which of the following best describes your household? - Other (please specify)	Open text response	
DATA.Q27r9oe = ReplicateRows(DATA_prepare.Q27r9oe,NCT*NAlt,1);

% Q28: Do you or a household member suffer from a long-term illness or disability?		Values: 0-1		
% 	0	Unchecked
% 	1	Checked
% 	[Q28r1]	No
% 	[Q28r2]	Yes - me
% 	[Q28r3]	Yes - household member
% 	[Q28r4]	Prefer not to say
DATA.Q28r1 = ReplicateRows(DATA_prepare.Q28r1,NCT*NAlt,1);
DATA.Q28r2 = ReplicateRows(DATA_prepare.Q28r2,NCT*NAlt,1);
DATA.Q28r3 = ReplicateRows(DATA_prepare.Q28r3,NCT*NAlt,1);
DATA.Q28r4 = ReplicateRows(DATA_prepare.Q28r4,NCT*NAlt,1);

% [Q29]: Does this illness/disability limit your daily activity?		Values: 1-2		
% 	1	No
% 	2	Yes
DATA.Q29 = ReplicateRows(DATA_prepare.Q29,NCT*NAlt,1);

% [Q30]: Does this illness/disability limit their daily activity?		Values: 1-2		
% 	1	No
% 	2	Yes
DATA.Q30 = ReplicateRows(DATA_prepare.Q30,NCT*NAlt,1);

% [Q31]: Which of the following best describes your current employment status?		Values: 1-13		
% 	1	Self-employed
% 	2	Employed full-time (30 hours per week or more)
% 	3	Employed part-time (8 - 29 hours per week)
% 	4	Employed working less than 8 hours a week
% 	5	Student
% 	6	Unemployed - seeking work
% 	7	Unemployed - not seeking work/other
% 	8	Looking after the home/children full-time
% 	9	Retired
% 	10	Unable to work due to temporary sickness
% 	11	Unable to work due to long-term sickness or disability
% 	12	Other (please specify)
% 	13	Prefer not to say
DATA.Q31 = ReplicateRows(DATA_prepare.Q31,NCT*NAlt,1);

%[Q31r12oe]: Which of the following best describes your current employment status? - Other (please specify)	Open text response	
DATA.Q31r12oe = ReplicateRows(DATA_prepare.Q31r12oe,NCT*NAlt,1);

% [Q32]: At what level did you complete your education? If you are still studying, which level best describes the highest level of education you have obtained until now?		Values: 1-9		
% 	1	O levels / CSEs / GCSEs (any grades)
% 	2	A levels / AS level / higher school certificate
% 	3	NVQ (Level 1 and 2). Foundation / Intermediate / Advanced GNVQ / HNC / HND
% 	4	Other qualifications (e.g. City and Guilds, RSA/OCR, BTEC/Edexcel)
% 	5	First degree (e.g. BA, BSc)
% 	6	Higher degree (e.g. MA, PhD, PGCE, post graduate certificates and diplomas)
% 	7	Professional qualifications (teacher, doctor, dentist, architect, engineer, lawyer, etc.)
% 	8	No qualifications
% 	9	Prefer not to say
DATA.Q32 = ReplicateRows(DATA_prepare.Q32,NCT*NAlt,1);

% [Q33]: Please can you indicate your total household income before tax and other deductions? Please note this information will be used to check that we have surveyed a range of people. It will not be possible to identify any particular individual or address in the results.		Values: 1-13		
% 	1	Up to £499Up to £5,999
% 	2	£500 - £1,083£6,000 - £12,999
% 	3	£1,084 - £1,583£13,000 - £18,999
% 	4	£1,584 - £2,166£19,000 - £25,999
% 	5	£2,167 - £2,666£26,000 - £31,999
% 	6	£2,667 - £3,000£32,000 - £35,999
% 	7	£3,001 - £3,500£36,000 - £41,999
% 	8	£3,501 - £4,000£42,000 - £47,999
% 	9	£4,001 - £5,333£48,000 - £63,999
% 	10	£5,334 - £7,999£64,000 - £95,999
% 	11	£8,000 and over£96,000 and over
% 	12	Don't know
% 	13	Prefer not to say
DATA.Q33 = ReplicateRows(DATA_prepare.Q33,NCT*NAlt,1);

% [Q34]: Which of the following best describes your ethnic group?		Values: 1-13		
% 	1	White British
% 	2	White Irish
% 	3	Mixed - White and Black Caribbean
% 	4	Mixed - White and Black African
% 	5	Mixed - White and Asian
% 	6	Indian
% 	7	Pakistani
% 	8	Bangladeshi
% 	9	Black Caribbean
% 	10	Black African
% 	11	Chinese
% 	12	Other (please specify)
% 	13	Prefer not to say
DATA.Q34 = ReplicateRows(DATA_prepare.Q34,NCT*NAlt,1);

%[Q34r12oe]: Which of the following best describes your ethnic group? - Other (please specify)	Open text response	
DATA.Q34r12oe = ReplicateRows(DATA_prepare.Q34r12oe,NCT*NAlt,1);

% Q35: Are you a member of any national or local environment or conservation organisations?		Values: 0-1		
% 	0	Unchecked
% 	1	Checked
% 	[Q35r1]	Royal Society for the Protection of Bids (RSPB)
% 	[Q35r2]	Green-peace
% 	[Q35r3]	WWF UK
% 	[Q35r4]	National Trust
% 	[Q35r5]	Friends of the Earth
% 	[Q35r6]	A local wildlife trust of environmental organization
% 	[Q35r7]	Other (please specify)
% 	[Q35r8]	None of these
DATA.Q35r1 = ReplicateRows(DATA_prepare.Q35r1,NCT*NAlt,1);
DATA.Q35r2 = ReplicateRows(DATA_prepare.Q35r2,NCT*NAlt,1);
DATA.Q35r3 = ReplicateRows(DATA_prepare.Q35r3,NCT*NAlt,1);
DATA.Q35r4 = ReplicateRows(DATA_prepare.Q35r4,NCT*NAlt,1);
DATA.Q35r5 = ReplicateRows(DATA_prepare.Q35r5,NCT*NAlt,1);
DATA.Q35r6 = ReplicateRows(DATA_prepare.Q35r6,NCT*NAlt,1);
DATA.Q35r7 = ReplicateRows(DATA_prepare.Q35r7,NCT*NAlt,1);
DATA.Q35r8 = ReplicateRows(DATA_prepare.Q35r8,NCT*NAlt,1);

%[Q35r7oe]: Are you a member of any national or local environment or conservation organisations? - Other (please specify)	Open text response	
DATA.Q35r7oe = ReplicateRows(DATA_prepare.Q35r7oe,NCT*NAlt,1);

% [Q36]: Considering all of the information that you have been given, overall, how easy or difficult was it to answer the questions in this survey?		Values: 1-6		
% 	1	Very easy
% 	2	Fairly easy
% 	3	Neither easy nor difficult
% 	4	Fairly difficult
% 	5	Very difficult
% 	6	Don't know / prefer not to say
DATA.Q36 = ReplicateRows(DATA_prepare.Q36,NCT*NAlt,1);

% Q37: Finally, did you think this survey was:		Values: 0-1		
% 	0	Unchecked
% 	1	Checked
% 	[Q37r1]	Interesting
% 	[Q37r2]	Easy
% 	[Q37r3]	Too long
% 	[Q37r4]	Difficult to understand
% 	[Q37r5]	Educational
% 	[Q37r6]	Unrealistic / not credible
% 	[Q37r98]	Other (please specify)
% 	[Q37r99]	None of these
DATA.Q37r1 = ReplicateRows(DATA_prepare.Q37r1,NCT*NAlt,1);
DATA.Q37r2 = ReplicateRows(DATA_prepare.Q37r2,NCT*NAlt,1);
DATA.Q37r3 = ReplicateRows(DATA_prepare.Q37r3,NCT*NAlt,1);
DATA.Q37r4 = ReplicateRows(DATA_prepare.Q37r4,NCT*NAlt,1);
DATA.Q37r5 = ReplicateRows(DATA_prepare.Q37r5,NCT*NAlt,1);
DATA.Q37r6 = ReplicateRows(DATA_prepare.Q37r6,NCT*NAlt,1);
DATA.Q37r98 = ReplicateRows(DATA_prepare.Q37r98,NCT*NAlt,1);
DATA.Q37r99 = ReplicateRows(DATA_prepare.Q37r99,NCT*NAlt,1);

%[Q37r98oe]: Finally, did you think this survey was: - Other (please specify)	Open text response	
DATA.Q37r98oe = ReplicateRows(DATA_prepare.Q37r98oe,NCT*NAlt,1);

% Timings: Timings for each question/section	Values: -99999999999999-999999999999999	
% 	[TimingsSectionA_Timer]
% 	[TimingsSectionB_Timer]
% 	[TimingsSectionC_Timer]
% 	[TimingsSectionD_Timer]
% 	[TimingsSectionE_Timer]
% 	[TimingsQ10_Timer]
% 	[TimingsQ11_Timer]
% 	[TimingsQ12_Timer]
% 	[TimingsQ13_Timer]
% 	[TimingsQ14_Timer]
% 	[TimingsChoice_Timer]
DATA.TimingsSectionA_Timer = ReplicateRows(DATA_prepare.TimingsSectionA_Timer,NCT*NAlt,1);
DATA.TimingsSectionB_Timer = ReplicateRows(DATA_prepare.TimingsSectionB_Timer,NCT*NAlt,1);
DATA.TimingsSectionC_Timer = ReplicateRows(DATA_prepare.TimingsSectionC_Timer,NCT*NAlt,1);
DATA.TimingsSectionD_Timer = ReplicateRows(DATA_prepare.TimingsSectionD_Timer,NCT*NAlt,1);
DATA.TimingsSectionE_Timer = ReplicateRows(DATA_prepare.TimingsSectionE_Timer,NCT*NAlt,1);

DATA.TimingsQ10_Timer = ReplicateRows(DATA_prepare.TimingsQ10_Timer,NCT*NAlt,1);
DATA.TimingsQ11_Timer = ReplicateRows(DATA_prepare.TimingsQ11_Timer,NCT*NAlt,1);
DATA.TimingsQ12_Timer = ReplicateRows(DATA_prepare.TimingsQ12_Timer,NCT*NAlt,1);
DATA.TimingsQ13_Timer = ReplicateRows(DATA_prepare.TimingsQ13_Timer,NCT*NAlt,1);
DATA.TimingsQ14_Timer = ReplicateRows(DATA_prepare.TimingsQ14_Timer,NCT*NAlt,1);

DATA.TimingsChoice_Timer = ReplicateRows(DATA_prepare.TimingsChoice_Timer,NCT*NAlt,1);


DATA.UUID18 = ReplicateRows(DATA_prepare.UUID18,NCT*NAlt,1);
DATA.UUIDFG = ReplicateRows(DATA_prepare.UUIDFG,NCT*NAlt,1);

DATA.Weights = ReplicateRows(DATA_prepare.Weights,NCT*NAlt,1);


%% ****************************  model specification  ****************************


% Recov:
% 0 - No change
% 1 - Minimal  -> Low       wildlife presence
% 2 - Low      -> Moderate  wilflife presence
% 3 - Moderate -> Full      wildlife presense

% Sites:
% 0 - N/A
% 1 - Small sites
% 2 - Medium sites
% 3 - Large sites

% habitat_presented
% 1 - Arable land 
% 2 - Blanket bog 
% 3 - Heathland 
% 4 - Lowland fens 
% 5 - Lowland hay meadows 
% 6 - Native mixed deciduous woodland 
% 7 - Semi-natural grassland 
% 8 - Upland oakwood 
% 9 - Wood pasture and parkland 
% 10 - Rivers 
% 11 - Coastal sand dunes 

% baseline 2w %:
DATA.Xa = [DATA.Recov==1, DATA.Recov==2, DATA.Recov==3, ...
    DATA.Sites==2, DATA.Sites==3, DATA.HabAmt_perc, -DATA.Cost/100];
DATA.Y = DATA.Choice;
EstimOpt.NamesA = {'Minimal -> Low wildlife presence', 'Low -> Moderate wildlife presence', 'Moderate -> Full wildlife presence', ...
    'Medium sites (vs. small)', 'Large sites (vs. small)', ...
    'Amount of this habitat targeted (100%)', '-Cost (100 GBP)'};
EstimOpt.ProjectName = 'defra_baseline2w';

% % habitat interactions:
% DATA.Xm = [DATA.habitat_presented==2,DATA.habitat_presented==3,DATA.habitat_presented==4, DATA.habitat_presented==5, DATA.habitat_presented==6, DATA.habitat_presented==7, DATA.habitat_presented==8,DATA.habitat_presented==9, DATA.habitat_presented==10, DATA.habitat_presented==11];
% EstimOpt.NamesM = {'Blanket bog (vs. Arable land)','Heathland (vs. Arable land)','Lowland fens (vs. Arable land)','Lowland hay meadows (vs. Arable land)',...
%     'Native mixed deciduous woodland  (vs. Arable land)','Semi-natural grassland (vs. Arable land)','Upland oakwood  (vs. Arable land)','Wood pasture and parkland (vs. Arable land)','Rivers (vs. Arable land)','Coastal sand dunes (vs. Arable land)'};
% EstimOpt.ProjectName = 'defra_2w_habitats';

% % baseline 2w ha:
% DATA.Xa = [DATA.Recov==1, DATA.Recov==2, DATA.Recov==3, ...
%     DATA.Sites==2, DATA.Sites==3, DATA.HabAmt_ha/1000, -DATA.Cost/100];
% DATA.Y = DATA.Choice;
% EstimOpt.NamesA = {'Minimal -> Low wildlife presence', 'Low -> Moderate wildlife presence', 'Moderate -> Full wildlife presence', ...
%     'Medium sites (vs. small)', 'Large sites (vs. small)', ...
%     'Amount of this habitat targeted (1000 ha)', '-Cost (100 GBP)'};
% EstimOpt.ProjectName = 'defra_baseline2w_ha';

% % habitat interactions:
% DATA.Xm = [DATA.habitat_presented==2,DATA.habitat_presented==3,DATA.habitat_presented==4, DATA.habitat_presented==5, DATA.habitat_presented==6, DATA.habitat_presented==7, DATA.habitat_presented==8,DATA.habitat_presented==9, DATA.habitat_presented==10, DATA.habitat_presented==11];
% EstimOpt.NamesM = {'Blanket bog (vs. Arable land)','Heathland (vs. Arable land)','Lowland fens (vs. Arable land)','Lowland hay meadows (vs. Arable land)',...
%     'Native mixed deciduous woodland  (vs. Arable land)','Semi-natural grassland (vs. Arable land)','Upland oakwood  (vs. Arable land)','Wood pasture and parkland (vs. Arable land)','Rivers (vs. Arable land)','Coastal sand dunes (vs. Arable land)'};
% 
% EstimOpt.ProjectName = 'defra_2w_habitats_ha';

% non-linear amount:
% dstats(DATA.HabAmt_perc,1)
% DATA.Xa = [DATA.Recov==1, DATA.Recov==2, DATA.Recov==3, ...
%     DATA.Sites==2, DATA.Sites==3,...
%     DATA.HabAmt_perc==0.05,DATA.HabAmt_perc==0.1,DATA.HabAmt_perc==0.125,DATA.HabAmt_perc==0.15,DATA.HabAmt_perc==0.2,DATA.HabAmt_perc==0.25,DATA.HabAmt_perc==0.375,DATA.HabAmt_perc==0.5,...
%     -DATA.Cost/100];
% DATA.Y = DATA.Choice;
% EstimOpt.NamesA = {'Minimal -> Low wildlife presence', 'Low -> Moderate wildlife presence', 'Moderate -> Full wildlife presence', ...
%     'Medium sites (vs. small)', 'Large sites (vs. small)', ...
%     'Amount of this habitat targeted (5% vs. 2.5%)','Amount of this habitat targeted (10% vs. 2.5%)','Amount of this habitat targeted (12.5% vs. 2.5%)','Amount of this habitat targeted (15% vs. 2.5%)','Amount of this habitat targeted (20% vs. 2.5%)','Amount of this habitat targeted (25% vs. 2.5%)','Amount of this habitat targeted (37.5% vs. 2.5%)','Amount of this habitat targeted (50% vs. 2.5%)',...
%     '-Cost (100 GBP)'};
% EstimOpt.ProjectName = 'defra_nl_amount';

% scale*amount interaction:
% dstats(DATA.HabAmt_perc,1)
% DATA.Xa = [DATA.Recov==1, DATA.Recov==2, DATA.Recov==3, ...
%     DATA.Sites==2, DATA.Sites==3, ...
%     (DATA.Recov==1).*DATA.HabAmt_perc, (DATA.Recov==2).*DATA.HabAmt_perc,(DATA.Recov==3).*DATA.HabAmt_perc,...
%     -DATA.Cost/100];
% DATA.Y = DATA.Choice;
% EstimOpt.NamesA = {'Minimal -> Low wildlife presence', 'Low -> Moderate wildlife presence', 'Moderate -> Full wildlife presence', ...
%     'Medium sites (vs. small)', 'Large sites (vs. small)', ...
%     'Minimal -> Low wildlife presence * Amount of this habitat targeted (100%)', 'Low -> Moderate wildlife presence * Amount of this habitat targeted (100%)', 'Moderate -> Full wildlife presence * Amount of this habitat targeted (100%)', ...
%     '-Cost (100 GBP)'};
% EstimOpt.ProjectName = 'defra_scale_amount_int';



%% ****************************  specifying input ****************************


DATA.filter = true(size(DATA.Choice));
DATA.filter = ~isnan(DATA.UUID18) & ~isnan(DATA.UUIDFG); % Drop respondents that are under 18 (UUID18; n=250 I think?) and the focus group respondents (UUIDFG; n=132? Something like that).

INPUT.Y = DATA.Y(DATA.filter); 
INPUT.Xa = DATA.Xa(DATA.filter,:);
% INPUT.Xm = DATA.Xm(DATA.filter,:);

INPUT.W = DATA.Weights(DATA.filter); 


%% ****************************  sample characteristics ****************************


EstimOpt.NCT = 12;
EstimOpt.NAlt = 3;
EstimOpt.NP = length(INPUT.Y)/EstimOpt.NCT/EstimOpt.NAlt;


%% **************************** estimation and optimization options ****************************


[INPUT,Results,EstimOpt,OptimOpt] = DataCleanDCE(INPUT,EstimOpt);

% EstimOpt.eps = 1.e-9; % overall precision level
% 
% load('OptimOpt1');

EstimOpt.NRep = 1e4; % number of draws for numerical simulation
% % OptimOpt.MaxIter = 1e0; % maximum number of iterations
% 
% % OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
% % EstimOpt.NumGrad = 1; % 1
% % OptimOpt.GradObj = 'off'; % 'off'
% % OptimOpt.FinDiffType = 'central'; % 'forward'
% % OptimOpt.Hessian = 'user-supplied'; % 'off'
% % EstimOpt.HessEstFix = 1; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian
% % EstimOpt.ApproxHess = 0;
% 
% EstimOpt.RealMin = 0;


%% ****************************     MNL     ****************************



% EstimOpt.HessEstFix = 1; % 0 = use optimization Hessian, 1 = use jacobian-based (BHHH) Hessian, 2 - use high-precision jacobian-based (BHHH) Hessian 3 - use numerical Hessian

EstimOpt.WTP_space = 1;

% baseline 2w: 
B_backup = [2.46636499137199;4.2927924801849;5.44417660233527;0.00888089849533399;0.110062888504008;2.0713936077397;0.33946591934407];

% habitat interactions 2w: 
% B_backup = [2.4709041319374667189;4.0429332828751078921;5.1568122033971457441;0.049878659115273477065;0.092041536971030624148;1.9302470941529408233;0.35939258746667407385;-0.41475671997228180654;-0.12445535866035215;0.0039903284074766006556;0.10833319027989403438;0.30802986681953820058;-0.070829268864279884554;-0.033594916464269250489;0.017100817897467063267;0.38001556025441063369;0.15544857845693260678;-0.17087349814861199637;-0.1901022993431354946;0.081235514605982842018;-0.023048607899901232127;0.082807450611179958533;0.20931856262186904205;0.33959267818356603241;-0.060263815741112065905;0.0055620931490188351906;-0.33677959617375280432;-0.016472701075797853421;0.59309836150943651134;0.52940344331194466587;0.64874844600351244672;-0.3102188219741283115;-0.11784077296176354022;-0.1503307352003018238;-0.0244330027603064312;-0.32296243116150069596;0.081538464037301583431;0.10550077541953953308;0.00083285205780928451419;0.023149485200638305477;2.4816833578826198448;-0.024985291438246192558;-0.34266551593931893205;-0.032592405933886479941;0.17771088249097483081;0.091652771248658684855;0.006288096734243146281;1.9655100204738329417;-0.020543873778336930319;-0.26659088900812327738;-0.011746260474650103631;-0.053708649256840365616;-0.19329216021573275563;-0.09891108227181058532;0.074417364414492134794;0.0039319303017857302335;-0.22755545646681829552;0.49482428732918742886;0.12661647972704004328;-0.24575277258618241638;-0.22096099033080160434;0.50644068121045893172;-0.01008353167291347989;0.5933576239756369608;1.0275100238332135483;1.3952345548702596023;0.45884746378250940158;0.75692061578780456266;-0.19026920022786053543;-0.054722428866905696576;-0.60215806816129502366;0.15672624912533741903;0.17384610669030381436;0.84895912145040364205;0.90906028212146727796;-0.3132789402294914316;-0.060062272545060260698];

% baseline 2w ha: 
% B_backup = [2.7248034537300758195;4.4890966593784602523;5.6082615488742444398;-0.023700025030225054662;0.10714509985663192571;0.0053635497014505413224;0.34228309600466189089];

% habitat interactions 2w ha: 
% B_backup = [B_backup; zeros(size(INPUT.Xa,2).*size(INPUT.Xm,2),1)];
% B_backup = [2.4971542880288581934;4.0641653327278710606;5.180874053923884226;0.045969898083793078736;0.087662286426187674415;0.0035418437186918614798;0.35919050793826173829;-0.43741368621736576028;-0.14197926813152000891;-0.017334573270349908081;0.11159497394218938215;0.31119229000728348966;0.0042475792091346978654;-0.033367174296299241532;-0.0029193370042570306391;0.3607185608441985103;0.13409815949392986378;-0.16579200822758027867;-0.18683736161838193635;0.0037179536583840885368;-0.022652053919511926788;0.057024612737901061998;0.18852655527245545941;0.31607603610278867112;-0.056614201757428782302;0.0096856087405118307387;0.076133411506119311185;-0.016288353995421566561;0.57747749201897491389;0.51323364639199398063;0.63195864550050417918;-0.30669707530513962146;-0.11227932694806729008;0.070840782573789271015;-0.024165656389636958751;-0.3344173330221975049;0.070304730795278297384;0.090413737936937924045;0.0045310754191717249675;0.026687984025126561954;0.0021903744875700079456;-0.024463833473369871879;-0.36417087595087033192;-0.051618540026388382014;0.15637869046067739975;0.0955606111412049819;0.011026792446890709004;0.0034216515949273642629;-0.020254774674821084124;-0.29878779310154035542;-0.034884943874203745706;-0.077609442447046608771;-0.18723767371958932593;-0.097282092248009294222;0.04194284090547675814;0.0038188079381170120415;-0.26251134778432594352;0.46678608135212201757;0.096319147669938667833;-0.23809058167949953555;-0.21415062940352366794;0.010689872591343413041;-0.0099196359594896053868;0.56977558372059977643;1.0152246497949930504;1.3722159973296828017;0.46087608741544572322;0.75562031538811480491;0.054570237209637675879;-0.054301973307835767202;-0.62805697402016436026;0.13619183443821655799;0.15046553947362992232;0.85287884370596545036;0.9135689629590036942;0.07727050937758274396;-0.05988268895308500317];

% non-linear amount:
% B_backup = [2.06284259836990;3.96385622359789;5.12951093554709;0.0942946132686744;0.192879810897916;0.517956031789378;0.674111229727248;0.520660681987532;0.938163756975827;1.14594262019280;0.972237127140973;1.03722533946446;1.21132576184066;0.336496433004241];

% scale*amount interaction:
% B_backup = [2.30804631164722;4.54525901538829;5.29128118731232;-0.0322674240283090;0.108149176416934;2.83722679421599;0.843252735879386;2.84475500212309;0.339743820206811];

OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.MNL = MNL(INPUT,Results,EstimOpt,OptimOpt);


%% ****************************     MXL_d     ****************************

EstimOpt.FullCov = 0;

% baseline 2w: 
B_backup = [1.6911770022330465668;3.2736959246263399592;4.5833302906896715712;0.066152026979091740899;0.15010782910682254898;1.9540126892243347978;-0.33371879742517479572;2.0250866068341260018;2.0772955463499611639;2.999495303853798589;0.068182431213778704748;0.74929955146978566294;3.3448566699187649576;1.1326854332546276716];

% % habitat interactions 2w:
% B_backup = [1.5762300879062625114;2.7675855677605727401;3.9457625487100100869;0.073898071581341914782;0.23705959021530503827;2.450931234522543356;-0.21866895291235266541;1.9179982833587636559;2.1438562138869241558;3.2468989284074738499;0.033900793665079906325;0.72729642156064500114;4.4864533086537079498;0.99123317566989799232;-0.26946688292584225488;0.036876546049932769356;0.22493943922973833049;0.25268871919140578619;0.3760234486870965287;-0.61518201495800517442;-0.33428539698067571972;-0.0033000795706386097488;0.4298221691196674743;0.1964056566949585747;-0.0041821719675000082125;-0.19687333067293755207;-0.81894378394300337831;-0.12409329037485364311;0.4243612602739628703;0.68863297649718302473;0.72971741023120950675;-0.1027060213888246476;-0.11815709894161322002;-1.0115150109266453082;-0.22518720180936238617;0.41692850194572655953;0.63127896163601515678;0.48110009380077922714;-0.14081912829718581937;-0.24043681817986453653;-1.0876277878768039198;-0.14105568484736999424;-0.35729372772799217195;0.038898344445923584145;-0.096345653483984786125;-0.10371291334749903335;-0.24259803492235984534;1.3638470757828971003;-0.017693111030839585307;-0.27102077049287875976;-0.05071997513067365293;0.0050612360382103647669;0.087773572277063047209;-0.10250097827681742269;0.70821459098449257574;0.031087500292118909473;0.057522549385065492589;0.27412788927907189018;0.056090595832770852314;-0.1205712391248620341;-0.24480432088558748416;-0.45891567327187982039;0.041681661573577846736;-0.13346898534869469644;0.75587531323686851792;0.22337695190154252534;-0.17746647819473068686;-0.38591103951205141254;-0.1095819917481572342;-0.12952771404126253718;0.084069155255331234899;0.50418846180184395678;0.74964966731739668404;0.45907164729760108202;0.60581735081267706899;-0.90692154826400828149;-0.26388892927153595469;-0.60937476565741721668;0.1150767521236991664;0.26650971752900787237;0.74774437856377362621;0.72842451579066014311;-0.72940098859139490539;-0.40329224155666726359];
% EstimOpt.NRep = 1e3; % number of draws for numerical simulation

% baseline 2w ha: 
% B_backup = [6.9151761467124632787;8.8434221036561897478;10.179625355280101928;0.059061935111171082191;0.11164676451031226023;0.0071553029250109988907;-1.2888081105685924577;1.8675197288535543283;-0.025574054502971142744;1.7658282202552646467;-0.36016214845810218259;0.56772498947220273458;-0.0012525231179609031592;1.8367086028548753607];

% habitat interactions 2w ha: 
% B_backup = [B_backup; zeros(size(INPUT.Xa,2).*size(INPUT.Xm,2),1)];
% B_backup = [5.1852570750453752524;6.8668391909518220473;8.2313483870531261033;0.098330656372355509931;0.256340690233227253;0.0053417406289040210779;-1.0881412160413410639;1.9246218105320807013;0.024092456776838893501;1.7326382284104278053;-0.15047410910249453542;0.58971044099371294767;0.00063834896023540828487;1.8149904068787470202;0.46949658345629585732;0.72679243429976325697;0.89424012710897982092;0.26005927107214887872;0.034540530721882636844;0.0010933308524603256324;-0.24050068693975057421;1.1657180444214878534;1.6231222178225461761;1.3661783423368858426;-0.11359395335262838234;-0.3735122123408874395;0.00055448982756671504925;-0.16826121351235637236;1.3021475024548363475;1.3622331877138700307;1.3678534297065467751;-0.079341523198323382626;-0.24439803837605300618;0.063264739234354078201;-0.13212587285361410139;1.1997480135512734734;1.2800289271432063742;1.1730951737346755337;-0.11010514025420324591;-0.22204983192190630437;0.053937272054490080497;-0.14834119304234233416;0.81757786092465667416;1.1640047937296711389;1.1717541646240177133;-0.11356181677688212406;-0.01385999100012386652;0.00056910609357607266738;-0.11744531853442878588;-0.26113040578299862027;0.061521483113286423716;0.0783939732957231461;0.04452299285508520138;-0.32729012997010781261;0.00070711198461949256039;0.01027106299276974756;0.11607490389181028545;0.28729102158129388167;0.16510164649984021557;-0.21266377274876149817;-0.19059227818835425272;0.03750953984930851387;0.023346698628534653941;1.0263352508188940959;1.7140772279647655818;1.4900196798913689644;-0.18625596057758128654;-0.19972363653614100265;0.010015026838052432795;-0.15435778639590827521;4.1992441002857008314;4.6458580783533767189;5.0432649118178085601;0.5408435866369278866;0.32876616119511814507;0.054942034377948013868;-0.32292049945710726844;3.2204736571925267619;4.1063849022283740453;4.3466526726478988962;0.79937265830370118902;0.35107353248524764133;0.093324748390805664644;-0.41658572953665101712];
% EstimOpt.NRep = 1e3; % number of draws for numerical simulation

% non-linear amount:
% B_backup = [1.32226348385586;3.03374218599531;4.23534910684809;0.142878056347084;0.106626065607789;0.448530770867331;0.716288221114233;0.632706746294639;0.835911639360745;0.855967020150955;0.843551113938880;0.867634434964265;1.02510658524421;-0.326375771890646;2.28753631810032;2.27667289979867;3.36498576192968;0.111111245797766;0.537095429712956;0.0781270854129693;0.439140601212142;0.160031086008686;-0.0470293817685952;0.0178284278489962;0.381561431058542;0.0290315168388424;-0.0803284081849367;1.20221179842108];

% scale*amount interaction:
% B_backup = [1.79450810677930;3.39844691919687;4.40304513060382;0.0805708981478592;0.110543251457983;1.42753732621434;0.759166712008159;2.05991063926277;-0.307219603181992;2.23763586833083;2.34983671048094;3.37325117235726;0.164150782647075;0.476662729763686;0.234109975853380;0.283573609786825;-1.63369146967369;1.21347620474856];


OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt); 

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.MXL_d = MXL(INPUT,Results,EstimOpt,OptimOpt); 


%% ****************************     MXL     ****************************


EstimOpt.FullCov = 1;

% baseline 2w:
B_backup = [7.4074656446432856072;9.5673559458599335414;11.075520799053007792;0.14649589985904612477;0.049622958225127640486;1.9292220933249866199;-0.53847839420638143881;4.7564803175544074776;6.100818695693453364;7.1427990141677035751;0.0239475328772998583;-0.02618865545483587895;0.9827522514403589371;-1.2700991227145737028;0.97847609889892606105;1.6447094761865828083;0.17817334281388563788;0.1387763168520403656;-0.090194986937280133277;-0.037378341515944248086;-0.021841272830391767618;0.047488946022393722923;0.03386106341762541716;0.041437283261149635005;0.04588884007309932539;-0.0070583459914679309161;-0.046069236186156992119;0.25963080181020786741;-0.57639098370521157921;0.020829133297970734567;0.18800593702731135948;-0.81784279540148396848;-0.043469286086529911051;0.21232941126842047597;0.035153107321372431548];

% habitat interactions 2w: 
% B_backup = [7.2427697393835277495;9.2274660373955530446;10.656918562444731791;0.096068755132978522759;0.058710289115875460841;1.5113490841399543907;-0.41629438650111366682;5.1034129647309773858;6.6209199079569316382;7.8194312923917435043;0.030540192850089027676;0.020531670024774708455;0.93417123686363179313;-1.2653732179514187717;0.7605134023794392073;1.2980523735516174;0.17876376740781577124;0.17702343809954154685;-0.16582202519881367175;0.15221473245932706786;0.054097355941693361925;-0.019129557439436926242;0.025747017120294448528;-0.1131505580955531054;-0.14979784292375875054;-0.0081385975648090684575;-0.017922858173077784588;0.31476385644704446554;-0.95957751902478116701;0.035395539350239452192;-0.18061913123399631065;0.22504017407258106176;0.032872572100778714443;-0.069254727053056366892;0.12656162210050905803;-0.33147607796105171829;-0.22382199419714990385;-0.12512769151299316106;0.11963930105862845277;0.10001493417830570332;0.33055293153087839109;-0.17683946269907377391;-0.037981937286894230932;-0.0041713502299800600537;-0.11000823742845142195;-0.012493413560912719137;-0.055696503022896193258;0.26033088270147830556;-0.019619867508347092755;-0.011081870882665153383;0.0094871454454521515071;0.050288336003638606231;-0.041253517068014930458;-0.037610021128934099932;0.1962500675660324434;-0.092579620935115458646;0.028156530184519103915;-0.018687879699231999681;-0.090754861590602128874;-0.0078758368580393693648;-0.02244996229887184086;0.14665860242927342139;-0.0025687141220731382098;-0.12715652675740396482;-0.11732200491962503641;-0.093985370738914386646;-0.0087531976977379133409;-0.01436720776566594894;1.0939519648617603842;0.028477266916216412845;-0.21299191921724092147;-0.16616100687564980265;-0.10481854484347549739;0.13018346880974246416;0.072950902375000237221;0.38667879511772440537;0.026316183036239920723;-0.054667690249957552606;-0.078354695616720163032;-0.14640950782501666039;-0.01381184841635078181;-0.023785074621826628183;0.29465756304192958215;0.068623243264861175095;-0.25216398732268291427;-0.046663450150123748916;-0.16793044173426735499;-0.073226145317612748031;0.068242758091998242942;0.78855185469396338327;-0.015288804806251493976;-0.098896381981546377382;-0.064784965340837288861;0.0028414308337416818544;0.15668324758489385329;0.13663565627808496594;0.19370711195777121083;-0.054308141283086190654;-0.27841566662719346992;-0.21583611821946269149;-0.021267695658352459009;0.21260414785453440878;0.22734565812925558448;0.19356961830530022839;-0.14841882471778650165];

% baseline 2w ha:
% B_backup = [8.1702617778730282083;10.345362163025315638;11.845966303152303922;0.1095828946565487183;0.0982028580923928196;0.0068337673592723307756;-0.56687132357017155915;5.2580108866427046621;6.7464298499845156698;7.8876923000186565105;0.051230898823695580901;0.036241590179456870113;0.0040603420774562061502;-1.3510793892004864247;0.66232574407952848894;1.1309125197669565033;0.13874562067632489892;0.13684420970987085098;-0.00059222625208141331087;0.19027819071240947846;-0.070705819776637326979;0.00022324077815481002175;0.051727329911891717362;0.0001952089202404364975;-0.60416077575543525313;-0.065574115760217266402;-0.023655702603383312738;-0.00025975865808926229929;-0.50418316609865054811;0.048378231819784241197;-0.00061975540002207703174;0.5062344082529321998;0.00018834152876352140324;0.28677445168540699472;0.10004575692734918313];

% habitat interactions 2w ha: 
% B_backup = [B_backup; zeros(size(INPUT.Xa,2).*size(INPUT.Xm,2),1)];
% B_backup = [8.0166147394755729039;10.02378876811403785;11.528738943879423218;0.092987688253402520089;0.087952213754401986634;0.0046140345047551347551;-0.5191349233660927931;5.1171395086203137481;6.541120345809342318;7.6639932418976366435;-0.012650358056748009505;0.031540966947921432462;0.0034176474342451431017;-1.3129834921211223087;0.65681540658693271517;1.2049480478912006909;0.15543667434663638316;0.16518270471836313251;-0.0002603712296708486011;0.21599905689410209275;0.020225531821513393488;-0.027760059373480677064;-0.053191317577202960887;0.0007602123236149791255;-0.81659703155938023755;-0.030740577278970053487;-0.028245863861480188794;-9.7113564613100349663e-05;0.089998709451021438599;-0.073075891679888760333;0.00018113742801284955222;0.39643518462590593421;0.00029882594620999182802;0.13889777955950413646;0.048894705606406622944;-0.45421074510177167349;-0.29160080340698646406;-0.18400428237122390396;0.10455132602180697288;0.12064694473889864268;0.0017744036434758643072;-0.16128560615578091397;-0.18699175360167044624;-0.10133164265978363006;-0.17322373347819430833;-0.057810678120501224342;-0.026450526462656491489;0.00125873387935677007;-0.029220660881078452248;-0.18897545722023309511;-0.041312446962927899807;0.031409756184471254725;-0.072233643799115926365;-0.030802574801536780624;0.016986526842049418096;-0.083791042997814854654;-0.11485021945918701125;-0.13028412518920573504;-0.17800235773193731625;-0.030514939248822432749;-0.0045565203847236380313;0.018666797803364613278;0.0055073398656756431327;-0.29169561880045113966;-0.16989456112542278277;-0.16121084095724083629;-0.018305868819570560169;0.088127678892201868899;0.0021812930068743907608;0.018134456438292614194;-0.23883117226004277334;-0.11445688474413076574;-0.084463784478519363108;0.046734032678340531319;0.02384907357154945895;0.0011804395638138952349;0.005334398442292848791;-0.20574616993956740263;-0.15000381575512827803;-0.24806224096272855295;-0.10085286897044715149;-0.078513194936057351114;0.012489125211456149692;0.079246167286603580449;-0.43499723554275426807;-0.14329769024526661791;-0.30718972301767066835;-0.1009622553946802892;0.014349350449524232373;0.0059042780168292617257;0.0032184827877559211984;-0.23231442610084393485;-0.16964581649808374575;-0.076024185518747608437;0.11660227186856000903;0.080456641641630230466;0.0171506676426505765;-0.046712192845934072394;-0.57930353823988056661;-0.45290933662416354988;-0.27362138886543474481;0.24667200515650689052;0.27809520858573033575;0.02729645013347917204;-0.13899715906279910826];

% non-linear amount:
% B_backup = [7.51608763857545;9.61424113591064;11.1107705330479;0.164484912006304;0.0882979032438670;0.0361122186673012;0.369219539951938;0.425965210689449;0.584551564683332;0.732054045790313;0.632976461749311;0.852234158680466;0.903502865789408;-0.493383849315671;5.38951956678735;6.72546003869428;7.86107820699132;-0.0218297561771947;-0.0754301006729261;-0.322895439911142;-0.0524845168481190;-0.0192619438089624;0.121215026547824;0.115430242016360;0.126264752808557;0.192666074818383;0.207068445772048;-1.23239857943838;0.871655969573152;1.38830898491663;0.169109473143330;0.157388760769805;-0.173162442388032;-0.0154843733071974;-0.0214417165556678;-0.0758271142717043;0.0523618797637261;-0.224941465105215;-0.140589373578288;-0.128395061083102;-0.159695785516963;-0.112739522224191;0.0673110010706655;0.108305421308862;0.402672117761849;0.397913052650509;0.378380303431483;0.322537061822608;0.542180565705367;0.393104625710273;0.473847774566386;0.474887779442370;-0.666941299053631;0.00361762707187355;-0.0597957897726777;-0.129455944982829;0.00199267182040041;-0.101847144420502;-0.0700016886662861;-0.0659714135687311;-0.119865646019608;-0.105922909441954;-0.111373943150928;0.475100600168062;-0.0454680875293364;-0.00620335555794248;-0.0846712712351458;-0.0105191396139900;-0.0497052144362862;-0.104955900100004;-0.00999543159354718;0.00342989141993749;0.0631356951066360;-0.0821311799408493;-0.00204675160382505;0.0765490798874695;-0.00863410517632087;0.0230319216013358;-0.0954914398798373;0.0382037351290133;0.0720550031480780;-0.00123376311217343;-0.351857961580822;-0.0586098507188729;-0.0510253796514882;-0.00903631229852477;-0.0890526565990195;-0.0330641804133051;-0.0196950317067130;-0.0151654055079203;-0.453458896298690;-0.0377840057691921;0.0115723161998118;0.0737336113024152;-0.0268931454771128;-0.00801489132043991;-0.0527975172734551;0.0180808121164088;0.0126418525781230;-0.0742881223883174;-0.00912679828630237;0.0213913310876643;-0.0112440298527111;0.188011247298759;-0.0442925654400517;0.0237736309170421;-0.0244343149022515;-0.000104683026530860;-0.295423889156096;-0.0618499566752758;0.0209171203857107;0.0183489765552192;0.0862519928403680;-0.0220114524887727;0.0964203797925289;-0.395569157729339;0.00309277869179776;-0.0517794970526079;0.153437346888113];

% scale*amount interaction:
% B_backup = [7.33149817149033;9.72884605632185;10.7515276765779;0.0639149352340851;0.0462601491069050;2.15840099571561;0.733657108312172;3.22414789331759;-0.521131046987286;4.67815783695218;6.09125322164217;6.85130399324269;-0.0182752510600891;-0.0422485518368610;1.07884862544703;0.398535917224395;1.74782971827426;-1.29308360150040;1.25430392248935;1.83294093715806;0.108192055213370;0.100786104450803;0.620656721334833;-0.496807515199585;-0.109272157660141;0.0146177579011315;-0.0808332855577500;-0.000735516167694465;-0.0461686793576924;-0.148686120611117;-0.0771277050677900;0.475595058068739;-0.148363283345515;0.0777199830510690;-0.0237233948583189;-0.163080460646217;0.211085565585345;0.132467056491110;0.541420821766175;-0.0229861709508686;0.520226637699418;0.151167095818085;-0.138801990021308;0.148413767092314;-0.493848864059010;-0.296545836498964;-0.0569722355298537;0.596460279425105;-0.179203202671178;-0.216974662172662;0.495907122570508;0.121343013099892;-0.309273502008583;-0.0288582386030692];

OptimOpt.Algorithm = 'trust-region'; % 'quasi-newton';
OptimOpt.Hessian = 'user-supplied'; % 'off'

Results.MXL = MXL(INPUT,Results,EstimOpt,OptimOpt);

OptimOpt.Algorithm = 'quasi-newton';
OptimOpt.Hessian = 'off';

Results.MXL = MXL(INPUT,Results,EstimOpt,OptimOpt);

